
{ Program AlignTest / Unit TestFunctions / Include File CharPosAsmFuncs_X86_64.inc
  This file contains the i386 versions of the tested CharPos assembler functions.
  It is called by the unit Testfunctions.pas.

  Copyright (C) 2015 Ruediger Walter <rue.walter@web.de>

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


  ************************************************************************************
  *  For better comparability please don't edit this file when sharing the results!  *
  ***********************************************************************************}


const
  Revision_CharPosAsm_I386_IncFile = 'r1002-20150308';

{$OPTIMIZATION REGVAR}
{$ASMMODE att}

{ Register assignment (all subsequent functions):
      @S          (Pointer)   In    ESI
      c           (Byte)            DL
      CntDownS    (Word)            AX
      Result      (SmallInt)  Out   AX
      LenS        (Word)            CX       }

function CharPos_AsmDummy_NoLoop(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi             // Save ESI
            movl   s, %esi          // @s -> ESI (s handed over in EDX)
            movzbl c, %edx          // c -> DL   (c handed over in AL)
            xorl   %eax, %eax       // Clear EAX
            xorl   %ecx, %ecx       // Clear ECX
            addb   (%esi), %cl      // Length(s) -> CL   (XOR/ADD is faster than MOVZx/TEST sequence)
            jz     .LExit           // if Length(s) = 0 then exit with result = 0 in AX
            movw   %cx, %ax         // copy Length(s) -> AX
//          incw   %ax              // CntDownS in AX := Length(s)+1  ( not used in Version 1)
            incl   %esi             // @S[1] -> ESI                   (only used in Version 1)
            jmp    .LExit
  .balign 16
  .LResult: subw   %ax, %cx         // Match result := LenS in CX - CntDownS in AX + 1 -> AX
            movw   %cx, %ax         // Never reached
            incw   %ax
  .LExit:   popl   %esi             // Restore ESI
end;

{******************************  Asm functions Version 1:  ******************************}
{*****  Char comparison first, unconditional loop jump, countdown with 4 byte subw  *****}

function CharPos_Asm1_LoopStart_0(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi             // Save ESI
            movl   s, %esi          // @s -> ESI (s handed over in EDX)
            movzbl c, %edx          // c -> DL   (c handed over in AL)
            xorl   %eax, %eax       // Clear EAX
            xorl   %ecx, %ecx       // Clear ECX
            addb   (%esi), %cl      // Length(s) -> CL   (XOR/ADD is faster than MOV/CMP sequence)
            jz     .LExit           // if Length(s) = 0 then exit with result = 0 in AX
            movw   %cx, %ax         // copy Length(s) -> AX
            incl   %esi             // @S[1] -> ESI                   (only used in Version 1)
            jmp    .LLoop
    .balign 16                      // aligned to 16
  .LLoop:   cmpb   (%esi), %dl      // 2 Byte code 3a16
            je     .LResult         // 2 Byte code 74xx
            subw   $1, %ax          // 4 Byte code 662d0100
            jz     .LExit           // 2 Byte code 74xx
            incl   %esi             // 1 Byte code 46
            jmp    .LLoop           // 2 Byte code ebxx
    .balign 16
  .LResult: subw   %ax, %cx         // Match result := LenS in CX - CntDownS in AX + 1 -> AX
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi             // Restore ESI
end;

function CharPos_Asm1_LoopStart_1(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 1 byte NOP (never reached)
    .byte   0x90                                                                //  1 x NOP   }
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_2(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 2 byte NOP (never reached)
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_3(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 3 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_4(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 4 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_5(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 5 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_6(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 6 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_7(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 7 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_8(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 8 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_9(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 9 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_A(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 10 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_B(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 11 byte NOP (never reached)
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_C(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 12 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_D(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 13 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_E(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 14 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_F(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 16                      // 0 + 15 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_32(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 32                      // aligned to 32
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm1_LoopStart_64(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb   (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incl   %esi
            jmp    .LLoop
    .balign 64                      // aligned to 64
  .LLoop:   cmpb   (%esi), %dl
            je     .LResult
            subw   $1, %ax
            jz     .LExit
            incl   %esi
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

{******************************  Asm functions Version 2:  *****************************}
{*****  Char comparison last, unconditional loop jump, countdown with 4 byte subw  *****}

function CharPos_Asm2_LoopStart_0(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi               // Save ESI
            movl   s, %esi            // @s -> ESI (s handed over in EDX)
            movzbl c, %edx            // c -> DL   (c handed over in AL)
            xorl   %eax, %eax         // Clear EAX
            xorl   %ecx, %ecx         // Clear ECX
            addb  (%esi), %cl         // Length(s) -> CL   (XOR/ADD is faster than MOVZx/TEST sequence)
            jz     .LExit             // if Length(s) = 0 then exit with result = 0 in AX
            movw   %cx, %ax           // copy Length(s) -> AX
            incw   %ax                // CntDownS in AX := Length(s)+1  ( not used in Version 1)
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax            // 4 Byte code 662d0100
            jz     .LExit             // 2 Byte code 74xx
            incl   %esi               // 1 Byte code 46
            cmpb   (%esi), %dl        // 2 Byte code 3a16
            je     .LResult           // 2 Byte code 74xx
            jmp    .LLoop             // 2 Byte code ebxx
    .balign 16
  .LResult: subw   %ax, %cx           // Match result := LenS in CX - CntDownS in AX + 1 -> AX
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi               // Restore ESI
end;

function CharPos_Asm2_LoopStart_1(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 1 byte NOP (never reached)
    .byte   0x90                                                                //  1 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_2(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 2 byte NOP (never reached)
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_3(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 3 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_4(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 4 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_5(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 5 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_6(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 6 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_7(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 7 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_8(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 8 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_9(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 9 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_A(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 10 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_B(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 11 byte NOP (never reached)
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_C(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 12 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_D(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 13 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_E(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 14 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_F(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 15 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_32(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 32                        // align to 32
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm2_LoopStart_64(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 64                        // align to 64
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            je     .LResult
            jmp    .LLoop
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

{******************************  Asm functions Version 3a:  ****************************}
{******  Char comparison last, conditional loop jump, countdown with 4 byte subw  ******}

function CharPos_Asm3a_LoopStart_0(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi               // Save ESI
            movl   s, %esi            // @s -> ESI (s handed over in EDX)
            movzbl c, %edx            // c -> DL   (c handed over in AL)
            xorl   %eax, %eax         // Clear EAX
            xorl   %ecx, %ecx         // Clear ECX
            addb  (%esi), %cl         // Length(s) -> CL   (XOR/ADD is faster than MOVZx/TEST sequence)
            jz     .LExit             // if Length(s) = 0 then exit with result = 0 in AX
            movw   %cx, %ax           // copy Length(s) -> AX
            incw   %ax                // CntDownS in AX := Length(s)+1  ( not used in Version 1)
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax            // 4 Byte code 662d0100
            jz     .LExit             // 2 Byte code 74xx
            incl   %esi               // 1 Byte code 46
            cmpb   (%esi), %dl        // 2 Byte code 3a16
            jne    .LLoop             // 2 Byte code 75xx
            jmp    .LResult           // 2 Byte code ebxx
    .balign 16
  .LResult: subw   %ax, %cx           // Match result := LenS in CX - CntDownS in AX + 1 -> AX
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi               // Restore ESI
end;

function CharPos_Asm3a_LoopStart_1(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 1 byte NOP (never reached)
    .byte   0x90                                                                //  1 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_2(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 2 byte NOP (never reached)
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_3(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 3 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_4(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 4 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_5(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 5 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_6(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 6 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_7(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 7 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_8(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 8 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_9(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 9 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_A(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 10 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_B(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 11 byte NOP (never reached)
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_C(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 12 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_D(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 13 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_E(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 14 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_F(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 15 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_32(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 32                        // aligned to 32
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3a_LoopStart_64(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 64                        // aligned to 64
  .LLoop:   subw   $1, %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

{******************************  Asm functions Version 3b:  *****************************}
{*******************  Same as Version 3a, countdown with 2 byte decw  *******************}

function CharPos_Asm3b_LoopStart_0(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi               // Save ESI
            movl   s, %esi            // @s -> ESI (s handed over in EDX)
            movzbl c, %edx            // c -> DL   (c handed over in AL)
            xorl   %eax, %eax         // Clear EAX
            xorl   %ecx, %ecx         // Clear ECX
            addb  (%esi), %cl         // Length(s) -> CL   (XOR/ADD is faster than MOVZx/TEST sequence)
            jz     .LExit             // if Length(s) = 0 then exit with result = 0 in AX
            movw   %cx, %ax           // copy Length(s) -> AX
            incw   %ax                // CntDownS in AX := Length(s)+1  ( not used in Version 1)
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   decw   %ax                // 2 Byte code 6648
            jz     .LExit             // 2 Byte code 74xx
            incl   %esi               // 1 Byte code 46
            cmpb   (%esi), %dl        // 2 Byte code 3a16
            jne    .LLoop             // 2 Byte code 75xx
            jmp    .LResult           // 2 Byte code ebxx
    .balign 16
  .LResult: subw   %ax, %cx           // Match result := LenS in CX - CntDownS in AX + 1 -> AX
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi               // Restore ESI
end;

function CharPos_Asm3b_LoopStart_1(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 1 byte NOP (never reached)
    .byte   0x90                                                                //  1 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_2(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 2 byte NOP (never reached)
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_3(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 3 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_4(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 4 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_5(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 5 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_6(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 6 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_7(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 7 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_8(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 8 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_9(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 9 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_A(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 10 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_B(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 11 byte NOP (never reached)
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_C(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 12 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_D(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 13 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_E(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 14 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_F(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // 0 + 15 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_32(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 32                        // aligned to 32
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3b_LoopStart_64(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 64                        // aligned to 64
  .LLoop:   decw   %ax
            jz     .LExit
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

{******************************  Asm functions Version 3c:  *****************************}
{******  Same as Version 3a, loop start aligned to 16, internal (runthrough) NOPs  ******}

function CharPos_Asm3c_InsideLoop_0(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi               // Save ESI
            movl   s, %esi            // @s -> ESI (s handed over in EDX)
            movzbl c, %edx            // c -> DL   (c handed over in AL)
            xorl   %eax, %eax         // Clear EAX
            xorl   %ecx, %ecx         // Clear ECX
            addb  (%esi), %cl         // Length(s) -> CL   (XOR/ADD is faster than MOVZx/TEST sequence)
            jz     .LExit             // if Length(s) = 0 then exit with result = 0 in AX
            movw   %cx, %ax           // copy Length(s) -> AX
            incw   %ax                // CntDownS in AX := Length(s)+1  ( not used in Version 1)
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax            // 4 Byte code 662d0100
            jz     .LExit             // 2 Byte code 74xx
            incl   %esi               // 1 Byte code 46
            cmpb   (%esi), %dl        // 2 Byte code 3a16
            jne    .LLoop             // 2 Byte code 75xx
            jmp    .LResult           // 2 Byte code ebxx
    .balign 16
  .LResult: subw   %ax, %cx           // Match result := LenS in CX - CntDownS in AX + 1 -> AX
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi               // Restore ESI
end;

function CharPos_Asm3c_InsideLoop_1(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x90                                                                //  1 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_2(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x90                                                          //  2 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_3(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_4(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_5(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_6(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_7(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_8(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_9(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_A(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_B(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_C(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_D(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_E(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_F(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_32(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
            .balign 32                        // aligned to 32
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

function CharPos_Asm3c_InsideLoop_64(c : Char; const s : ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            movl   s, %esi
            movzbl c, %edx
            xorl   %eax, %eax
            xorl   %ecx, %ecx
            addb  (%esi), %cl
            jz     .LExit
            movw   %cx, %ax
            incw   %ax
            jmp    .LLoop
    .balign 16                        // aligned to 16
  .LLoop:   subw   $1, %ax
            jz     .LExit
    .balign 64                        // aligned to 64
            incl   %esi
            cmpb   (%esi), %dl
            jne    .LLoop
            jmp    .LResult
    .balign 16
  .LResult: subw   %ax, %cx
            movw   %cx, %ax
            incw   %ax
  .LExit:   popl   %esi
end;

{******************************  Asm functions Asm4a_NoLoop:  ***************************}
{************  Dummy without doing anything but jumping to NOPed Exit (ret)  ************}

function CharPos_Asm4a_NoLoop_ExitStart_0(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // Aligned to 16
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_1(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 1 byte NOP (never reached)
    .byte   0x90                                                                //  1 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_2(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 2 byte NOP (never reached)
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_3(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 3 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_4(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 4 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_5(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 5 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_6(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 6 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_7(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 7 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_8(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 8 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_9(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 9 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_A(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 10 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_B(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 11 byte NOP (never reached)
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_C(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 12 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_D(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 13 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_E(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 14 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_F(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 16                      // 0 + 15 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_32(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 32                      // Align 32 (never reached)
  .LExit:
end;

function CharPos_Asm4a_NoLoop_ExitStart_64(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            jmp    .LExit
    .balign 64                      // Align 64 (never reached)
  .LExit:
end;

{******************************  Asm functions Asm4b_NoLoop:  ***************************}
{**************  Same as Version 4a, but with PUSH / NOPed Exit (POP RET)  **************}

function CharPos_Asm4b_NoLoop_ExitStart_0(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // Aligned to 16
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_1(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 1 byte NOP (never reached)
    .byte   0x90                                                                //  1 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_2(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 2 byte NOP (never reached)
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_3(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 3 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_4(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 4 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_5(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 5 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_6(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 6 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_7(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 7 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_8(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 8 byte NOP (never reached)
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_9(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 9 byte NOP (never reached)
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_A(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 10 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_B(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 11 byte NOP (never reached)
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_C(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 12 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_D(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 13 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_E(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 14 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_F(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 16                      // 0 + 15 byte NOP (never reached)
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_32(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 32                      // Align 32 (never reached)
  .LExit:   popl   %esi
end;

function CharPos_Asm4b_NoLoop_ExitStart_64(c: Char; const s: ShortString): SmallInt; assembler; register; nostackframe;
asm
            pushl  %esi
            jmp    .LExit
    .balign 64                      // Align 64 (never reached)
  .LExit:   popl   %esi
end;

procedure InitFuncGroups;
begin
  with CharPosFuncGroup_Asm1 do
  begin
    Func[0]  := @CharPos_Asm1_LoopStart_0;
    Func[1]  := @CharPos_Asm1_LoopStart_1;
    Func[2]  := @CharPos_Asm1_LoopStart_2;
    Func[3]  := @CharPos_Asm1_LoopStart_3;
    Func[4]  := @CharPos_Asm1_LoopStart_4;
    Func[5]  := @CharPos_Asm1_LoopStart_5;
    Func[6]  := @CharPos_Asm1_LoopStart_6;
    Func[7]  := @CharPos_Asm1_LoopStart_7;
    Func[8]  := @CharPos_Asm1_LoopStart_8;
    Func[9]  := @CharPos_Asm1_LoopStart_9;
    Func[10] := @CharPos_Asm1_LoopStart_A;
    Func[11] := @CharPos_Asm1_LoopStart_B;
    Func[12] := @CharPos_Asm1_LoopStart_C;
    Func[13] := @CharPos_Asm1_LoopStart_D;
    Func[14] := @CharPos_Asm1_LoopStart_E;
    Func[15] := @CharPos_Asm1_LoopStart_F;
    Func[16] := @CharPos_Asm1_LoopStart_32;
    Func[17] := @CharPos_Asm1_LoopStart_64;
    Name     := 'CharPos_Asm1_LoopStart_';
    Text[0]  := '*****************************  Asm functions Version 1:  *****************************';
    Text[1]  := '   Char comparison first, unconditional loop jump, countdown with 4 byte subw:';
    Text[2]  := '   *****************************************************************';
    Text[3]  := '   *             .balign 16                                        *';
    Text[4]  := '   *             >> n bytes NOP <<                                 *';
    Text[5]  := '   *   .LLoop:   cmpb    (%esi), %dl    // 2 Byte Code 3a16        *';
    Text[6]  := '   *             je      .LResult       // 2 Byte Code 74xx        *';
    Text[7]  := '   *             subw    $1, %ax        // 4 Byte Code 662d0100    *';
    Text[8]  := '   *             jz      .LExit         // 2 Byte Code 74xx        *';
    Text[9]  := '   *             inc     %esi           // 1 Byte Code 46          *';
    Text[10] := '   *             jmp     .LLoop         // 2 Byte Code ebxx        *';
    Text[11] := '   *****************************************************************';
  end;
  with CharPosFuncGroup_Asm2 do
  begin
    Func[0]  := @CharPos_Asm2_LoopStart_0;
    Func[1]  := @CharPos_Asm2_LoopStart_1;
    Func[2]  := @CharPos_Asm2_LoopStart_2;
    Func[3]  := @CharPos_Asm2_LoopStart_3;
    Func[4]  := @CharPos_Asm2_LoopStart_4;
    Func[5]  := @CharPos_Asm2_LoopStart_5;
    Func[6]  := @CharPos_Asm2_LoopStart_6;
    Func[7]  := @CharPos_Asm2_LoopStart_7;
    Func[8]  := @CharPos_Asm2_LoopStart_8;
    Func[9]  := @CharPos_Asm2_LoopStart_9;
    Func[10] := @CharPos_Asm2_LoopStart_A;
    Func[11] := @CharPos_Asm2_LoopStart_B;
    Func[12] := @CharPos_Asm2_LoopStart_C;
    Func[13] := @CharPos_Asm2_LoopStart_D;
    Func[14] := @CharPos_Asm2_LoopStart_E;
    Func[15] := @CharPos_Asm2_LoopStart_F;
    Func[16] := @CharPos_Asm2_LoopStart_32;
    Func[17] := @CharPos_Asm2_LoopStart_64;
    Name     := 'CharPos_Asm2_LoopStart_';
    Text[0]  := '*****************************  Asm functions Version 2:  *****************************';
    Text[1]  := '   Char comparison last, unconditional loop jump, countdown with 4 byte subw:';
    Text[2]  := '   *****************************************************************';
    Text[3]  := '   *             .balign 16                                        *';
    Text[4]  := '   *             >> n bytes NOP <<                                 *';
    Text[5]  := '   *   .LLoop:   subw    $1, %ax        // 4 Byte Code 662d0100    *';
    Text[6]  := '   *             jz      .LExit         // 2 Byte Code 74xx        *';
    Text[7]  := '   *             inc     %esi           // 1 Byte Code 46          *';
    Text[8]  := '   *             cmpb    (%esi), %dl    // 2 Byte Code 3a16        *';
    Text[9]  := '   *             je      .LResult       // 2 Byte Code 74xx        *';
    Text[10] := '   *             jmp     .LLoop         // 2 Byte Code ebxx        *';
    Text[11] := '   *****************************************************************';
  end;
  with CharPosFuncGroup_Asm3a do
  begin
    Func[0]  := @CharPos_Asm3a_LoopStart_0;
    Func[1]  := @CharPos_Asm3a_LoopStart_1;
    Func[2]  := @CharPos_Asm3a_LoopStart_2;
    Func[3]  := @CharPos_Asm3a_LoopStart_3;
    Func[4]  := @CharPos_Asm3a_LoopStart_4;
    Func[5]  := @CharPos_Asm3a_LoopStart_5;
    Func[6]  := @CharPos_Asm3a_LoopStart_6;
    Func[7]  := @CharPos_Asm3a_LoopStart_7;
    Func[8]  := @CharPos_Asm3a_LoopStart_8;
    Func[9]  := @CharPos_Asm3a_LoopStart_9;
    Func[10] := @CharPos_Asm3a_LoopStart_A;
    Func[11] := @CharPos_Asm3a_LoopStart_B;
    Func[12] := @CharPos_Asm3a_LoopStart_C;
    Func[13] := @CharPos_Asm3a_LoopStart_D;
    Func[14] := @CharPos_Asm3a_LoopStart_E;
    Func[15] := @CharPos_Asm3a_LoopStart_F;
    Func[16] := @CharPos_Asm3a_LoopStart_32;
    Func[17] := @CharPos_Asm3a_LoopStart_64;
    Name     := 'CharPos_Asm3a_LoopStart_';
    Text[0]  := '*****************************  Asm functions Version 3a:  ****************************';
    Text[1]  := '   Char comparison last, conditional loop jump, countdown with 4 byte subw:';
    Text[2]  := '   *****************************************************************';
    Text[3]  := '   *             .balign 16                                        *';
    Text[4]  := '   *             >> n bytes NOP <<                                 *';
    Text[5]  := '   *   .LLoop:   subw    $1, %ax        // 4 Byte Code 662d0100    *';
    Text[6]  := '   *             jz      .LExit         // 2 Byte Code 74xx        *';
    Text[7]  := '   *             inc     %esi           // 1 Byte Code 46          *';
    Text[8]  := '   *             cmpb    (%esi), %dl    // 2 Byte Code 3a16        *';
    Text[9]  := '   *             jne     .LLoop         // 2 Byte Code 75xx        *';
    Text[10] := '   *             jmp     .LResult       // 2 Byte Code ebxx        *';
    Text[11] := '   *****************************************************************';
  end;
  with CharPosFuncGroup_Asm3b do
  begin
    Func[0]  := @CharPos_Asm3b_LoopStart_0;
    Func[1]  := @CharPos_Asm3b_LoopStart_1;
    Func[2]  := @CharPos_Asm3b_LoopStart_2;
    Func[3]  := @CharPos_Asm3b_LoopStart_3;
    Func[4]  := @CharPos_Asm3b_LoopStart_4;
    Func[5]  := @CharPos_Asm3b_LoopStart_5;
    Func[6]  := @CharPos_Asm3b_LoopStart_6;
    Func[7]  := @CharPos_Asm3b_LoopStart_7;
    Func[8]  := @CharPos_Asm3b_LoopStart_8;
    Func[9]  := @CharPos_Asm3b_LoopStart_9;
    Func[10] := @CharPos_Asm3b_LoopStart_A;
    Func[11] := @CharPos_Asm3b_LoopStart_B;
    Func[12] := @CharPos_Asm3b_LoopStart_C;
    Func[13] := @CharPos_Asm3b_LoopStart_D;
    Func[14] := @CharPos_Asm3b_LoopStart_E;
    Func[15] := @CharPos_Asm3b_LoopStart_F;
    Func[16] := @CharPos_Asm3b_LoopStart_32;
    Func[17] := @CharPos_Asm3b_LoopStart_64;
    Name     := 'CharPos_Asm3b_LoopStart_';
    Text[0]  := '*****************************  Asm functions Version 3b:  ****************************';
    Text[1]  := '   Same as Version 3a, countdown with 3 byte decw:';
    Text[2]  := '   *****************************************************************';
    Text[3]  := '   *             .balign 16                                        *';
    Text[4]  := '   *             >> n bytes NOP <<                                 *';
    Text[5]  := '   *   .LLoop:   decw    %ax            // 2 Byte Code 6648        *';
    Text[6]  := '   *             jz      .LExit         // 2 Byte Code 74xx        *';
    Text[7]  := '   *             inc     %esi           // 1 Byte Code 46          *';
    Text[8]  := '   *             cmpb    (%esi), %dl    // 2 Byte Code 3a16        *';
    Text[9]  := '   *             jne     .LLoop         // 2 Byte Code 75xx        *';
    Text[10] := '   *             jmp     .LResult       // 2 Byte Code ebxx        *';
    Text[11] := '   *****************************************************************';
  end;
  with CharPosFuncGroup_Asm3c do
  begin
    Func[0]  := @CharPos_Asm3c_InsideLoop_0;
    Func[1]  := @CharPos_Asm3c_InsideLoop_1;
    Func[2]  := @CharPos_Asm3c_InsideLoop_2;
    Func[3]  := @CharPos_Asm3c_InsideLoop_3;
    Func[4]  := @CharPos_Asm3c_InsideLoop_4;
    Func[5]  := @CharPos_Asm3c_InsideLoop_5;
    Func[6]  := @CharPos_Asm3c_InsideLoop_6;
    Func[7]  := @CharPos_Asm3c_InsideLoop_7;
    Func[8]  := @CharPos_Asm3c_InsideLoop_8;
    Func[9]  := @CharPos_Asm3c_InsideLoop_9;
    Func[10] := @CharPos_Asm3c_InsideLoop_A;
    Func[11] := @CharPos_Asm3c_InsideLoop_B;
    Func[12] := @CharPos_Asm3c_InsideLoop_C;
    Func[13] := @CharPos_Asm3c_InsideLoop_D;
    Func[14] := @CharPos_Asm3c_InsideLoop_E;
    Func[15] := @CharPos_Asm3c_InsideLoop_F;
    Func[16] := @CharPos_Asm3c_InsideLoop_32;
    Func[17] := @CharPos_Asm3c_InsideLoop_64;
    Name     := 'CharPos_Asm3c_InsideLoop_';
    Text[0]  := '*****************************  Asm functions Version 3c:  ****************************';
    Text[1]  := '   Same as Version 3a, loop start aligned to 16, internal (runthrough) NOPs:';
    Text[2]  := '   *****************************************************************';
    Text[3]  := '   *             .balign 16                                        *';
    Text[4]  := '   *   .LLoop:   subw    $1, %ax        // 4 Byte Code 662d0100    *';
    Text[5]  := '   *             jz      .LExit         // 2 Byte Code 74xx        *';
    Text[6]  := '   *             >> n bytes NOP <<                                 *';
    Text[7]  := '   *             inc     %esi           // 1 Byte Code 46          *';
    Text[8]  := '   *             cmpb    (%esi), %dl    // 2 Byte Code 3a16        *';
    Text[9]  := '   *             jne     .LLoop         // 2 Byte Code 75xx        *';
    Text[10] := '   *             jmp     .LResult       // 2 Byte Code ebxx        *';
    Text[11] := '   *****************************************************************';
  end;
  with CharPosFuncGroup_Asm4a do
  begin
    Func[0]  := @CharPos_Asm4a_NoLoop_ExitStart_0;
    Func[1]  := @CharPos_Asm4a_NoLoop_ExitStart_1;
    Func[2]  := @CharPos_Asm4a_NoLoop_ExitStart_2;
    Func[3]  := @CharPos_Asm4a_NoLoop_ExitStart_3;
    Func[4]  := @CharPos_Asm4a_NoLoop_ExitStart_4;
    Func[5]  := @CharPos_Asm4a_NoLoop_ExitStart_5;
    Func[6]  := @CharPos_Asm4a_NoLoop_ExitStart_6;
    Func[7]  := @CharPos_Asm4a_NoLoop_ExitStart_7;
    Func[8]  := @CharPos_Asm4a_NoLoop_ExitStart_8;
    Func[9]  := @CharPos_Asm4a_NoLoop_ExitStart_9;
    Func[10] := @CharPos_Asm4a_NoLoop_ExitStart_A;
    Func[11] := @CharPos_Asm4a_NoLoop_ExitStart_B;
    Func[12] := @CharPos_Asm4a_NoLoop_ExitStart_C;
    Func[13] := @CharPos_Asm4a_NoLoop_ExitStart_D;
    Func[14] := @CharPos_Asm4a_NoLoop_ExitStart_E;
    Func[15] := @CharPos_Asm4a_NoLoop_ExitStart_F;
    Func[16] := @CharPos_Asm4a_NoLoop_ExitStart_32;
    Func[17] := @CharPos_Asm4a_NoLoop_ExitStart_64;
    Name     := 'CharPos_Asm4a_NoLoop_ExitStart_';
    Text[0]  := '*******************************  Asm functions Version 4a:  ****************************';
    Text[1]  := '   Dummy doing nothing else but jumping to NOPed Exit (RET):';
    Text[2]  := '   ***************************************';
    Text[3]  := '   *   asm                               *';
    Text[4]  := '   *               jmp     .LExit        *';
    Text[5]  := '   *               .balign 16            *';
    Text[6]  := '   *               >> n bytes NOP <<     *';
    Text[7]  := '   *     .LExit:                         *';
    Text[8]  := '   *   end;                              *';
    Text[9]  := '   ***************************************';
  end;
  with CharPosFuncGroup_Asm4b do
  begin
    Func[0]  := @CharPos_Asm4b_NoLoop_ExitStart_0;
    Func[1]  := @CharPos_Asm4b_NoLoop_ExitStart_1;
    Func[2]  := @CharPos_Asm4b_NoLoop_ExitStart_2;
    Func[3]  := @CharPos_Asm4b_NoLoop_ExitStart_3;
    Func[4]  := @CharPos_Asm4b_NoLoop_ExitStart_4;
    Func[5]  := @CharPos_Asm4b_NoLoop_ExitStart_5;
    Func[6]  := @CharPos_Asm4b_NoLoop_ExitStart_6;
    Func[7]  := @CharPos_Asm4b_NoLoop_ExitStart_7;
    Func[8]  := @CharPos_Asm4b_NoLoop_ExitStart_8;
    Func[9]  := @CharPos_Asm4b_NoLoop_ExitStart_9;
    Func[10] := @CharPos_Asm4b_NoLoop_ExitStart_A;
    Func[11] := @CharPos_Asm4b_NoLoop_ExitStart_B;
    Func[12] := @CharPos_Asm4b_NoLoop_ExitStart_C;
    Func[13] := @CharPos_Asm4b_NoLoop_ExitStart_D;
    Func[14] := @CharPos_Asm4b_NoLoop_ExitStart_E;
    Func[15] := @CharPos_Asm4b_NoLoop_ExitStart_F;
    Func[16] := @CharPos_Asm4b_NoLoop_ExitStart_32;
    Func[17] := @CharPos_Asm4b_NoLoop_ExitStart_64;
    Name     := 'CharPos_Asm4b_NoLoop_ExitStart_';
    Text[0]  := '*******************************  Asm functions Version 4b:  *****************************';
    Text[1]  := '   Same as Version 4a, but with PUSH / NOPed Exit (POP RET):';
    Text[2]  := '   ***************************************';
    Text[3]  := '   *   asm                               *';
    Text[4]  := '   *               pushl   %esi          *';
    Text[5]  := '   *               jmp     .LExit        *';
    Text[6]  := '   *               .balign 16            *';
    Text[7]  := '   *               >> n bytes NOP <<     *';
    Text[8]  := '   *     .LExit:   popl    %esi          *';
    Text[9]  := '   *   end;                              *';
    Text[10] := '   ***************************************';
  end;
end;



{ for copy & paste:

    .balign 16                      // 0 +  byte NOP (never reached)
    .byte   0x66, 0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00    // 11 x NOP
    .byte   0x66, 0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00          // 10 x NOP
    .byte   0x66, 0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                //  9 x NOP
    .byte   0x0F, 0x1F, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00                      //  8 x NOP
    .byte   0x0F, 0x1F, 0x80, 0x00, 0x00, 0x00, 0x00                            //  7 x NOP
    .byte   0x66, 0x0F, 0x1F, 0x44, 0x00, 0x00                                  //  6 x NOP
    .byte   0x0F, 0x1F, 0x44, 0x00, 0x00                                        //  5 x NOP
    .byte   0x0F, 0x1F, 0x40, 0x00                                              //  4 x NOP
    .byte   0x0F, 0x1F, 0x00                                                    //  3 x NOP
    .byte   0x66, 0x90                                                          //  2 x NOP
    .byte   0x90                                                                //  1 x NOP

}
