
{ Program AlignTest / Unit Main

  Copyright (C) 2015 Ruediger Walter <rue.walter@web.de>

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}

unit Main;

{$mode objfpc}{$H+}
{ $DEFINE Developing}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Buttons, EpikTimer,
  EditBtn, StdCtrls, ExtCtrls, Spin, CPU, SortableHashList, DbgTimer, md5, TestFunctions;


const
  Version       = '0.1.0';
  Revision_Main = 'r1002-20150317';

type

  TStatus = (st_SearchFiles, st_ReadFiles, st_NoFiles, st_Stat, st_Test, st_CanTest, st_Done);

  TLogTag = (tge_OpSys, tge_CPUArchitecture, tgb_CPUIDSupport, tgs_CPUName, tgs_CPUVendorId,
             tgl_CPUMaxStdFunc, tgl_CPUMaxExtFunc, tgl_CPURawId, tgl_CPUFamily, tgl_CPUModel,
             tgl_CPUStepping, tgl_CPUBrandId, tgl_CPUExtBrandId, tgl_CPURawStdFlags_eax,
             tgl_CPURawStdFlags_ebx, tgl_CPURawStdFlags_ecx, tgl_CPURawStdFlags_edx,
             tgl_CPURawExtFlags_eax, tgl_CPURawExtFlags_ebx, tgl_CPURawExtFlags_ecx,
             tgl_CPURawExtFlags_edx, tgl_OptimizationLevel, tgl_DebugInfo, tgv_VersionInfo,
             tgl_NumFiles, tgl_NumStrings, tgl_NumRuns, tgl_NumChars, tgi_CumulatedResult,
             tgi_CharComparisons, tgi_Matches, tgi_Mismatches, tgi_NumCalls, tgf_DummyTime,
             tgf_ReferenceTime, tgs_FuncVersion, tgl_FuncLoopStart, tgb_FuncError, tgf_FuncRunTime,
             tgf_StartTime, tgf_TotalTestTime);

  TOpSys  = (os_Linux, os_Win32, os_win64, os_Unknown);

  TArch   = (ar_X86_64, ar_I386, ar_NotSupperted);

  TVersionInfo = Array[1..4, 1..2] of String;

  TCPUInfo = record
    Architecture    : TArch;
    OpSys           : TOpSys;
    CPUIDSupport    : Boolean;
    Name,
    VendorID        : ShortString;
    MaxStdFunc,
    MaxExtFunc,
    RawId,
    Family,
    Model,
    Stepping,
    BrandId,
    ExtBrandId,
    RawStdFlags_eax,         // Raw results from Standard function 1h (Feature flags)
    RawStdFlags_ebx,         // which might be of some interest in the future
    RawStdFlags_ecx,
    RawStdFlags_edx,
    RawExtFlags_eax,         // Raw results from Extended function 80000001h
    RawExtFlags_ebx,
    RawExtFlags_ecx,
    RawExtFlags_edx : LongWord;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Label5 : TLabel;
    Label6 : TLabel;
    RunsSpinEdit : TSpinEdit;
    MaxStringsSpinEdit : TSpinEdit;
    StatusLabel : TLabel;
    StartBtn : TBitBtn;
    CloseBtn : TBitBtn;
    DirEdit : TDirectoryEdit;
    Label1 : TLabel;
    Label2 : TLabel;
    Label3 : TLabel;
    Label4 : TLabel;
    CPUMemo : TMemo;
    StatMemo : TMemo;
    ResMemo : TMemo;
    procedure DirEditAcceptDirectory(Sender : TObject; var Value : String);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MaxStringsSpinEditChange(Sender : TObject);
    procedure StartBtnClick(Sender: TObject);
  private
    AlignTestPath,
    LogFileName,
    LoadedPath     : String;
    HashList       : TSortableHashList;
    IsTesting,
    IsReading,
    StopTesting,
    StopReading    : Boolean;
    NumFiles       : Integer;
    CPUInfo        : TCPUInfo;
    VersionInfo    : TVersionInfo;
    LogList        : TStringList;
    StartTime,
    ElapsedTime    : TDateTime;
    CompOptLevel,
    CompDebug,
    TotalFuncNum,
    CurrentFuncNum : Integer;
    procedure AddTag(SL: TStrings; LT: TLogTag; Value: String);
    procedure AddTag(SL: TStrings; LT: TLogTag; Value: Boolean);
    procedure AddTag(SL: TStrings; LT: TLogTag; Value: Int64);
    procedure AddTag(SL: TStrings; LT: TLogTag; Value: Extended; Num: integer = 0);
    procedure AddTag(SL: TStrings; LT: TLogTag; VI: TVersionInfo);
    procedure ReadCPUInfo;
    procedure Welcome(AtStart: Boolean);
    procedure WriteLogFile;
    procedure ReadData(Path: String);
    procedure ClearData;
    procedure SetStatus(st: TStatus; Appendix: String = '');
    function  FuncTest(CharPosFunc: TCharPosFunc; Version: String; LoopStart: Integer = -1): Extended;
    procedure FuncGroupTest(Group: TCharPosFuncGroup);
  end;

var
  Form1 : TForm1;

implementation

{$R *.lfm}

const
  {$ifdef LINUX}           InitialPath = '/usr/share/lazarus/';
  {$else} {$ifdef WINDOWS} InitialPath = 'c:\lazarus';
          {$else}          InitialPath = '';
          {$endif WINDOWS}
  {$endif LINUX}
  DefaultLoops = 4;
  FileMask     = '*.pas;*.pp;*.inc;*.lpr;*.dpr';
  SearchMask   : ShortString = 'AaBbCcDdEe_\7$%^| QT';
  OpSysName    : Array[TOpSys] of String = ('Linux', 'Win32', 'Win64', 'Unknown');
  ArchName     : Array[TArch]  of String = ('x86_64', 'i386', 'Not supported');


function IntToSepStr(i: Int64; Width: integer = 0): ShortString;  { Formats to "1,234,567,890" }
var                                 // Same as IntToStr, but with ThousandSeparator
  k : integer;                      // Replacing the cryptic "format('%*.0n',[i/1])"
  s : ShortString;                  // right-aligned, if Width is specified and > Length
begin
  Result := '';
  s := IntToStr(i);
  while Length(s) > 3 do
  begin
    k := Length(s) - 2;
    Result := ThousandSeparator + copy(s, k, 3) + Result;
    Delete(s, k, 3);
  end;
  Result := s + Result;
  for k := 1 to Width - Length(Result) do
    Result := ' ' + Result;
end;

function SanitizeFileName(s: String): String;                     { Eliminate invalid chars }
var
  i : integer;
begin
  i := 1;
  while i <= Length(s) do
    if s[i] in ['+', '-', '_', '('..')', '0'..'9', 'A'..'Z', '['..']', 'a'..'z'] then inc(i)
    else if s[i] = ' ' then s[i] := '_'
         else Delete(s, i, 1);
  Result := s;
end;


{ TForm1 }

{ **************************  Constructor / Destructor  ************************** }

procedure TForm1.FormCreate(Sender : TObject);
var
  ConfigFileName,
  s  : String;
  SL : TStringList;
  i  : integer;
begin
  LoadedPath := '';
  ThousandSeparator := ',';
  IsTesting := false;
  IsReading := false;
  StopTesting := false;
  StopReading := false;
  LogList := TStringList.Create;
  ReadCPUInfo;
  s := CPUInfo.Name;          // prepare CPU-Name for log filename
  i := Pos(': ', s);
  if i > 0 then Delete(s, 1, i+1);
  s := SanitizeFileName(s);
  LogFileName := ChangeFileExt(Application.ExeName, '') + 'Result_(' + s + ').txt';
  AlignTestPath := ExtractFilePath(LogFileName);
  HashList := TSortableHashList.Create(CaseSensitive);
  if DirectoryExists(InitialPath) then DirEdit.Directory := InitialPath;
  {$ifdef Developing} RunsSpinEdit.Value := 1;
              {$else} RunsSpinEdit.Value := DefaultLoops; {$endif}

  Caption := Caption + Version;
  s := AlignTestPath + 'main.pas';                                // get version info
  if FileExists(s) then
  begin
    VersionInfo[1,1] := Revision_Main;
    VersionInfo[1,2] := MD5Print(MD5File(s));
  end;
  s := AlignTestPath + 'testfunctions.pas';
  if FileExists(s) then
  begin
    VersionInfo[2,1] := Revision_TestFunctions;
    VersionInfo[2,2] := MD5Print(MD5File(s));
  end;
  s := AlignTestPath + 'CharPosAsmFuncs_i386.inc';
  if FileExists(s) then
  begin
    VersionInfo[3,1] := Revision_CharPosAsmFuncs_I386;
    VersionInfo[3,2] := MD5Print(MD5File(s));
  end;
  s := AlignTestPath + 'CharPosAsmFuncs_X86_64.inc';
  if FileExists(s) then
  begin
    VersionInfo[4,1] := Revision_CharPosAsmFuncs_X86_64;
    VersionInfo[4,2] := MD5Print(MD5File(s));
  end;

  ConfigFileName := ChangeFileExt(Application.ExeName, '.lpi'); // read Compiler Options
  if not FileExists(ConfigFileName) then
  begin
    CompOptLevel := -1;
    CompDebug    := -1;
    CPUMemo.Append('Compiler options are unknown');
    exit;
  end;
  SL := TStringList.Create;
  SL.LoadFromFile(ConfigFileName);
  i := 0;
  while (i < SL.Count) and (Pos('<OptimizationLevel Value=', SL[i]) = 0) do inc(i);
  if i = SL.Count then CompOptLevel := 1
  else begin
    s := SL[i];
    i := Pos('="', s) + 2;
    if (i <= Length(s)) and (s[i] in ['0'..'9']) then CompOptLevel := StrToInt(s[i])
    else CompOptLevel := -1;
  end;
  i := 0;
  while (i < SL.Count) and (Pos('<GenerateDebugInfo Value="False"', SL[i]) = 0) do inc(i);
  CompDebug := integer(i = SL.Count);
  SL.Free;
  s := 'Compiled with Optimization Level ';
  if CompOptLevel = - 1 then s := s + 'Unknown' else s := s + IntToStr(CompOptLevel);
  s := s + ' and Debug-Info ';
  if CompDebug = 1 then s := s + 'ON' else s := s + 'OFF';
  CPUMemo.Append(s);
end;

procedure TForm1.FormDestroy(Sender : TObject);
begin
  HashList.Free;
  LogList.Free;
end;

{ ********************************  Data Loading  ******************************** }

procedure TForm1.FormActivate(Sender : TObject);
var
  s : String;
begin
  if Application.Terminated then exit;  // Don't read data if architecture isn't supported
  s := DirEdit.Directory;
  if s <> '' then DirEditAcceptDirectory(Self, s);
end;

procedure TForm1.DirEditAcceptDirectory(Sender : TObject; var Value : String);
begin
  if DirectoryExists(Value) and (Value <> LoadedPath) then
  begin
    StopReading := IsReading;
    ReadData(Value);
  end
  else Value := LoadedPath;
end;

procedure TForm1.MaxStringsSpinEditChange(Sender : TObject);
begin
  StopReading := IsReading;
  if not IsReading then ReadData(LoadedPath);
end;

procedure TForm1.ClearData;
begin
  HashList.Clear;
  HashList.Sort;         // clears HashList.SortArray
  StatMemo.Clear;
  ResMemo.Clear;
  StartBtn.Enabled := false;
  LoadedPath := '';
end;

procedure TForm1.ReadData(Path: String);
var
  FileList : TStringList;

  {Sub-}
  procedure FileLinesToHashList;
  var
    f, k    : LongInt;
    s       : ShortString;
    NewPath : String;
    c       : Char;
    SL      : TStringList;
  begin
    if FileList.Count = 0 then exit;
    SL := TStringList.Create;
    for f := 0 to FileList.Count - 1 do
    begin
      if HashList.Count >= MaxStringsSpinEdit.Value then break;
      if StopReading then
      begin
        ClearData;
        IsReading := false;
        StopReading := false;
        NewPath := DirEdit.Directory;
        DirEditAcceptDirectory(Self, NewPath);       // Recursion!
        exit;
      end;
      if f mod 50 = 0 then
        SetStatus(st_ReadFiles, IntToStr(f) + ' / ' + IntToStr(FileList.Count));
      Application.ProcessMessages;
      SL.Clear;
      SL.LoadFromFile(FileList[f]);
      for k := 0 to SL.Count - 1 do
      begin
        s := SL[k];                           // Conversion to ShortString
        if s = '' then continue;
        if HashList.FindIndexOf(s) < 0 then HashList.Add(s, self);
      end;               // see http://bugs.freepascal.org/view.php?id=26800
    end;
    NumFiles := f + 1;
    HashList.Add('', self);                   // also add an empty String to the hashlist ...
    for c := ' ' to '~' do
    begin                                     // ... and single chars for testing purposes
      s := c;
      if HashList.FindIndexOf(s) < 0 then HashList.Add(s, self)
    end;
    SL.Free;
  end;
  {/Sub}

begin
  if IsReading then exit;
  ClearData;
  Welcome(True);
  LoadedPath := Path;
  if LoadedPath = '' then Exit;
  SetStatus(st_SearchFiles);
  StatMemo.Append('Directory:                                   ' + LoadedPath);
  StatMemo.Append('Filename Mask:                               ' + FileMask);
  Application.ProcessMessages;
  FileList := FindAllFiles(Path, FileMask, true);
  NumFiles := 0;
  IsReading := true;
  FileLinesToHashList;
  If IsReading then
  begin
    StatMemo.Append('Files loaded:' + IntToSepStr(NumFiles, 43));
    StatMemo.Append('Strings loaded:' + IntToSepStr(HashList.Count, 41));
  end;
  StartBtn.Enabled := HashList.Count > 0;
  HashList.Sort;                        // enables faster access via HashList.SortArray;
  if HashList.Count = 0 then SetStatus(st_NoFiles) else SetStatus(st_CanTest);
  IsReading := false;
  FileList.Free;
end;

{ ********************************  User Interface  ****************************** }

procedure TForm1.SetStatus(st : TStatus; Appendix : String);
var
  s : String;
begin
  if st < st_CanTest then StatusLabel.Font.Color := clMaroon
  else StatusLabel.Font.Color := clGreen;
  case st of
    st_SearchFiles : s := 'Searching Files...';
    st_ReadFiles   : s := 'Reading Files...';
    st_NoFiles     : s := 'No files found!';
    st_Stat        : s := 'Grabbing statistics';
    st_Test        : s := 'Testing...';
    st_CanTest     : s := 'Ready for testing!';
    st_Done        : s := 'Done! See Logfile:';
  end;
  if Appendix <> '' then s := s + '  ' + Appendix;
  StatusLabel.Caption := s;
  Application.ProcessMessages;
end;

procedure TForm1.Welcome(AtStart: Boolean);
begin
  ResMemo.Append('');
  ResMemo.Append('');
  if AtStart then
  ResMemo.Append('    **********************************  What is it about ?  ************************************')
  else
  ResMemo.Append('    *************************************  All Done!  ******************************************');
  ResMemo.Append('    **                                                                                        **');
  ResMemo.Append('    **    The scope of this test program is to learn more about how to write fast and         **');
  ResMemo.Append('    **    efficient Free Pascal code for modern processors. One of the most interesting       **');
  ResMemo.Append('    **    questions in this context is: What have different CPUs in common in terms of        **');
  ResMemo.Append('    **    micro-runtime behaviour (e.g. related to code alignment) and what are their         **');
  ResMemo.Append('    **    differences? If we optimize some code on our own working machine, would this code   **');
  ResMemo.Append('    **    also work better on other computers? So the more result log files of different      **');
  ResMemo.Append('    **    processors could be evaluated, the more reliable the findings would be.             **');
  ResMemo.Append('    **                                                                                        **');
  ResMemo.Append('    **    Thus I would like to ask you to run this test (which will take about 12 minutes)    **');
  ResMemo.Append('    **    and to submit your result file(s):    ==>     mailto: rue.walter@web.de             **');
  ResMemo.Append('    **                                                                                        **');
  ResMemo.Append('    ************************  Thank you very much for your patience!  **************************');
end;

{ *******************************  CPU Information  ****************************** }

{$ASMMODE att}

procedure TForm1.ReadCPUInfo;
{ Remark: First I tried to use TSMBios for gaining CPU informations. But TSMBios needs
          root privileges on Linux, which makes it quite useless. It also seems to be
          buggy on Linux and raises an exception inside TSMBios.GetSMBiosTablesCount
          even with root privileges (uSMBIOS.pas line 4914). As this test program only
          deals with x86-architecture, all relevant informations should be accessible
          by the CPUID instruction anyway. }
type
  TRegId  = (eax, ebx, ecx, edx);
  TRegs   = Array[TRegId] of LongWord;
const
  OSNotsupportetMsg = 'Program AlignTest: Only x86 architecture is supported. ' +
                      'Program will halt now. Sorry!';
  NoCPUIDSupportMsg = 'Program AlignTest: No CPU info available! ' +
                      'Must be a fairly old processor? ';
  NoExtCPUIDMsg     = 'Program AlignTest: No CPU Name available! ' +
                      'Must be a fairly exotic processor? ';
  AddCPUInfoMsg     = 'Please add some information about the CPU manually to the ' +
                      'logfile (or to its filename) if sharing the results.';

var
  Regs    : TRegs;
  l       : LongWord;
  s       : String;
  IsAMD,
  IsIntel : Boolean;

  {Sub-} function GetCPUIDLeaf(FuncNum: LongWord): TRegs;
         var
           _eax, _ebx, _ecx, _edx : LongWord;
         begin
           asm
             {$ifdef CPUX86_64}
             pushq %rbx
             {$else}
             pushl %ebx
             {$endif}
             movl    FuncNum, %eax
             cpuid
             movl    %eax, _eax
             movl    %ebx, _ebx
             movl    %ecx, _ecx
             movl    %edx, _edx
             {$ifdef CPUX86_64}
             popq %rbx
             {$else}
             popl %ebx
             {$endif}
           end ['eax', 'ecx', 'edx'];
           Result[eax] := _eax;
           Result[ebx] := _ebx;
           Result[ecx] := _ecx;
           Result[edx] := _edx;
  {/Sub} end;

  {Sub-} function RegToStr(RegId : TRegId): ShortString;
         type
           T4Chars = Array[1..4] of char;
         var
           ca : T4Chars;
           i  : integer;
         begin                // tricky, because name string is possibly zero-terminated
           result := '';
           ca := T4Chars(Regs[RegId]);
           for i := 1 to 4 do
             if ca[i] <> #0 then result := result + ca[i];
  {/Sub} end;

begin
  { Operating System }
  {$ifdef LINUX}                   CPUInfo.OpSys := os_Linux;
  {$else} {$ifdef WIN32}           CPUInfo.OpSys := os_Win32;
          {$else} {$ifdef WIN64}   CPUInfo.OpSys := os_win64;
                  {$else}          CPUInfo.OpSys := os_Unknown;
                  {$endif WIN64}
          {$endif WIN32}
  {$endif LINUX}
  { Architecture & CPUID-Support}
  {$ifdef CPUX86_64}               CPUInfo.Architecture := ar_X86_64;
                                   CPUInfo.CPUIDSupport := true;
  {$else} {$ifdef CPUI386}         CPUInfo.Architecture := ar_I386;
                                   CPUInfo.CPUIDSupport := cpuid_support;
          {$else}                  CPUInfo.Architecture := ar_NotSupperted;
                                   CPUInfo.CPUIDSupport := false;
          {$endif CPUI386}
  {$endif CPUX86_64}
  { End of the line ? }
  CPUMemo.Append('Operating System : ' + OpSysName[CPUInfo.OpSys]);
  CPUMemo.Append('Architecture     : ' + ArchName[CPUInfo.Architecture]);
  if CPUInfo.Architecture = ar_NotSupperted then
  begin
    ShowMessage(OSNotsupportetMsg);
    Application.Terminate;
  end;
  if not CPUInfo.CPUIDSupport then
  begin
    ShowMessage(NoCPUIDSupportMsg + AddCPUInfoMsg);
    exit;
  end;
  { Vendor }
  Regs := GetCPUIDLeaf(0);
  CPUInfo.MaxStdFunc := Regs[eax];
  CPUInfo.VendorID := RegToStr(ebx) + RegToStr(edx) + RegToStr(ecx);
  IsAMD := CPUInfo.VendorID = 'AuthenticAMD';
  IsIntel := CPUInfo.VendorID = 'GenuineIntel';
  { CPUId - Family - Model - Stepping - BrandId - Feature Flags}
  if CPUInfo.MaxStdFunc >= 1 then
  begin
    Regs := GetCPUIDLeaf(1);
    CPUInfo.RawId := Regs[eax];
    l := Regs[eax] and $0FFF0FFF;  // masking reserved bits out
    CPUInfo.Stepping := l and $F;
    l := l shr 4;                     {--------Std. function #01: Bitmask in EAX -------------}
    CPUInfo.Model := l and $F;        { 31-28 | 27-20 | 19-16 | 15-12 | 11-08 | 07-04 | 03-00 }
    l := l shr 4;                     { Resvd | ExtFam| ExtMod| Resvd | Family| Model | Stepg }
    CPUInfo.Family := l and $F;       {-------------------------------------------------------}
    if CPUInfo.Family = $F then
    begin
      l := l shr 4;
      CPUInfo.Model := CPUInfo.Model + (l and $F0);
      l := l shr 8;
      CPUInfo.Family := CPUInfo.Family + l;
    end;
    CPUInfo.BrandId := Regs[ebx] and $FF;
    CPUInfo.RawStdFlags_eax := Regs[eax];
    CPUInfo.RawStdFlags_ebx := Regs[ebx];       // These feature flags might be of later interest
    CPUInfo.RawStdFlags_ecx := Regs[ecx];
    CPUInfo.RawStdFlags_edx := Regs[edx];
  end;
  { Extended functions }
  if not (IsAMD or IsIntel) then
  begin
    ShowMessage(NoExtCPUIDMsg + AddCPUInfoMsg);
    exit;
  end;
  Regs := GetCPUIDLeaf($80000000);
  CPUInfo.MaxExtFunc := Regs[eax];
  { Extended BrandId - Feature Flags}
  if CPUInfo.MaxExtFunc >= $80000001 then
  begin
    Regs := GetCPUIDLeaf($80000001);
    if IsAMD then CPUInfo.ExtBrandId := Regs[eax] and $FFF  // masking extended BrandId bits
    else CPUInfo.ExtBrandId := Regs[eax] and $FFFF;         // AMD has 12 bits, Intel 16 bits
    CPUInfo.RawExtFlags_eax := Regs[eax];
    CPUInfo.RawExtFlags_ebx := Regs[ebx];       // These feature flags might be of later interest
    CPUInfo.RawExtFlags_ecx := Regs[ecx];
    CPUInfo.RawExtFlags_edx := Regs[edx];
  end;
  { Processor name }
  if CPUInfo.MaxExtFunc >= $80000004 then
  begin
    Regs := GetCPUIDLeaf($80000002);
    CPUInfo.Name := RegToStr(eax) + RegToStr(ebx) + RegToStr(ecx) + RegToStr(edx);
    Regs := GetCPUIDLeaf($80000003);
    CPUInfo.Name := CPUInfo.Name + RegToStr(eax) + RegToStr(ebx) + RegToStr(ecx) + RegToStr(edx);
    Regs := GetCPUIDLeaf($80000004);
    CPUInfo.Name := CPUInfo.Name + RegToStr(eax) + RegToStr(ebx) + RegToStr(ecx) + RegToStr(edx);
  end;
  if CPUInfo.CPUIDSupport then s := 'True' else s := 'False';
  CPUMemo.Append('CPUID Support    : ' + s);
  if not CPUInfo.CPUIDSupport then exit;
  CPUMemo.Append('CPU Vendor       : ' + CPUInfo.VendorID);
  CPUMemo.Append('CPU Name         : ' + CPUInfo.Name);
  CPUMemo.Append('CPUID            : ' + IntToHex(CPUInfo.RawId, 1) + ' h');
  CPUMemo.Append('CPU Family       : ' + IntToStr(CPUInfo.Family));
  CPUMemo.Append('CPU Model        : ' + IntToStr(CPUInfo.Model));
  CPUMemo.Append('CPU Stepping     : ' + IntToStr(CPUInfo.Stepping));
end;

{ *******************************  Result Logging  ******************************* }

procedure TForm1.WriteLogFile;
var
  SL   : TStringList;
  i, k : integer;
  s    : String;
  SR   : TSearchRec;
const
  EmptyLine = '**                                                                 **';
begin
  if StopTesting then exit;
  {$IFDEF Developing} exit; {$ENDIF Developing}
  i := 1;
  while FileExists(LogFileName) do
  begin
    inc(i);
    k := Pos(')_#', LogFileName);
    if k > 0 then
    begin
      inc(k);
      delete(LogFileName, k, Length(LogFileName) - k - 3);
    end
    else k := Length(LogFileName) - 3;
    insert('_#' + IntToStr(i), LogFileName, k);
  end;
  SL := TStringList.Create;
  s:= FormatDateTime('yyyy/mm/dd - hh:nn:ss', StartTime);
  SL.Append('');
  SL.Append('************************  AlignTest LogFile  ************************');
  SL.Append(EmptyLine);
  SL.Append('**  This file was generated by AlignTest on ' + s + '  **');
  SL.Append(EmptyLine);
  SL.Append('**                      Please do not edit !                       **');
  SL.Append(EmptyLine);
  SL.Append('*********************************************************************');
  for i := 1 to 3 do SL.Append('');
  SL.Append('********************  CPU & OS Environment:  ********************');
  SL.Append('');
  For i := 0 to CPUMemo.Lines.Count - 1 do SL.Append(CPUMemo.Lines[i]);
  SL.Append('');
  SL.Append('***********************  Test Statistics:  **********************');
  SL.Append('');
  For i := 0 to StatMemo.Lines.Count - 1 do SL.Append(StatMemo.Lines[i]);
  SL.Append('');
  SL.Append('************************  Test Results:  ************************');
  SL.Append('');
  For i := 0 to ResMemo.Lines.Count - 1 do SL.Append(ResMemo.Lines[i]);
  For i := 1 to 5 do SL.Append('');
  SL.Append('------------ The following part is intended for machine reading ------------');
  AddTag(SL, tge_OpSys,              OpSysName[CPUInfo.OpSys]);
  AddTag(SL, tge_CPUArchitecture,    ArchName[CPUInfo.Architecture]);
  AddTag(SL, tgb_CPUIDSupport,       CPUInfo.CPUIDSupport);
  AddTag(SL, tgs_CPUName,            CPUInfo.Name);
  AddTag(SL, tgs_CPUVendorId,        CPUInfo.VendorID);
  AddTag(SL, tgl_CPUMaxStdFunc,      CPUInfo.MaxStdFunc);
  AddTag(SL, tgl_CPUMaxExtFunc,      CPUInfo.MaxExtFunc);
  AddTag(SL, tgl_CPURawId,           CPUInfo.RawId);
  AddTag(SL, tgl_CPUFamily,          CPUInfo.Family);
  AddTag(SL, tgl_CPUModel,           CPUInfo.Model);
  AddTag(SL, tgl_CPUStepping,        CPUInfo.Stepping);
  AddTag(SL, tgl_CPUBrandId,         CPUInfo.BrandId);
  AddTag(SL, tgl_CPUExtBrandId,      CPUInfo.ExtBrandId);
  AddTag(SL, tgl_CPURawStdFlags_eax, CPUInfo.RawStdFlags_eax);
  AddTag(SL, tgl_CPURawStdFlags_ebx, CPUInfo.RawStdFlags_ebx);
  AddTag(SL, tgl_CPURawStdFlags_ecx, CPUInfo.RawStdFlags_ecx);
  AddTag(SL, tgl_CPURawStdFlags_edx, CPUInfo.RawStdFlags_edx);
  AddTag(SL, tgl_CPURawExtFlags_eax, CPUInfo.RawExtFlags_eax);
  AddTag(SL, tgl_CPURawExtFlags_ebx, CPUInfo.RawExtFlags_ebx);
  AddTag(SL, tgl_CPURawExtFlags_ecx, CPUInfo.RawExtFlags_ecx);
  AddTag(SL, tgl_CPURawExtFlags_edx, CPUInfo.RawExtFlags_edx);
  AddTag(SL, tgl_OptimizationLevel,  CompOptLevel);
  AddTag(SL, tgl_DebugInfo,          CompDebug);
  AddTag(SL, tgv_VersionInfo,        VersionInfo);
  AddTag(SL, tgl_NumFiles,           NumFiles);
  AddTag(SL, tgl_NumStrings,         HashList.Count);
  AddTag(SL, tgl_NumRuns,            RunsSpinEdit.Value);
  AddTag(SL, tgl_NumChars,           Length(SearchMask));
  AddTag(SL, tgi_CumulatedResult,    CharPosStat.CumulatedResult);
  AddTag(SL, tgi_CharComparisons,    CharPosStat.CharComparisons);
  AddTag(SL, tgi_Matches,            CharPosStat.Matches);
  AddTag(SL, tgi_Mismatches,         CharPosStat.Mismatches);
  AddTag(SL, tgi_NumCalls,           CharPosStat.NumCalls);
  AddTag(SL, tgf_DummyTime,          CharPosStat.DummyTime);
  AddTag(SL, tgf_ReferenceTime,      CharPosStat.ReferenceTime);
  For i := 0 to LogList.Count - 1 do SL.Append(LogList[i]);
  SL.SaveToFile(LogFileName);
  SL.Free;
end;

procedure TForm1.AddTag(SL: TStrings; LT: TLogTag; Value: String);
var
  s : string;
begin
  s := '<' + IntToStr(Word(LT)) + '>';
  SL.Append(s + '=' + Value);
end;

procedure TForm1.AddTag(SL: TStrings; LT: TLogTag; Value: Boolean);
var
  s : string;
begin
  s := '<' + IntToStr(Word(LT)) + '>';
  SL.Append(s + '=' + IntToStr(Word(Value)));
end;

procedure TForm1.AddTag(SL: TStrings; LT: TLogTag; Value: Int64);
var
  s : string;
begin
  s := '<' + IntToStr(Word(LT)) + '>';
  SL.Append(s + '=' + IntToStr(Value));
end;

procedure TForm1.AddTag(SL: TStrings; LT: TLogTag; Value: Extended; Num: integer = 0);
var
  s : string;
begin
  s := '<' + IntToStr(Word(LT)) + '>';
  if num > 0 then s := s + '#' + IntToStr(num);
  SL.Append(s + '=' + FloatToStr(Value));
end;

procedure TForm1.AddTag(SL: TStrings; LT: TLogTag; VI: TVersionInfo);
begin
  SL.Append('<' + IntToStr(Word(LT)) + '>#Ver_Main' + '=' + VI[1,1]);
  SL.Append('<' + IntToStr(Word(LT)) + '>#MD5_Main' + '=' + VI[1,2]);
  SL.Append('<' + IntToStr(Word(LT)) + '>#Ver_TestFunctions' + '=' + VI[2,1]);
  SL.Append('<' + IntToStr(Word(LT)) + '>#MD5_TestFunctions' + '=' + VI[2,2]);
  SL.Append('<' + IntToStr(Word(LT)) + '>#Ver_IncI386' + '=' + VI[3,1]);
  SL.Append('<' + IntToStr(Word(LT)) + '>#MD5_IncI386' + '=' + VI[3,2]);
  SL.Append('<' + IntToStr(Word(LT)) + '>#Ver_IncX86_64' + '=' + VI[4,1]);
  SL.Append('<' + IntToStr(Word(LT)) + '>#MD5_IncX86_64' + '=' + VI[4,2]);
end;

{ **************************  The Lion's Den:  Testing  ************************** }

function TForm1.FuncTest(CharPosFunc : TCharPosFunc;   { Function error debugging here }
                         Version: String; LoopStart: Integer = -1): Extended;
var
  i, iRun, iString, iChar : integer;
  s : String;
  CheckSum : Int64;
  LoopTimePart : Extended;

  {Sub-} procedure DebugError;  { For debugging purposes on error detection }
         var
           St             : ShortString;
           Ch             : Char;
           LenS           : Byte;
           iSt, iCh, Cnt  : integer;
           RefResult,
           ErrResult      : SmallInt;
         begin
           for iSt := 0 to High(HashList.SortArray) do
             for iCh := 1 to Length(SearchMask) do
             begin
               St := HashList.SortArray[iSt]^;
               Ch := SearchMask[iCh];
               LenS := Length(St);
               RefResult := Pos(Ch, St);
               ErrResult := CharPosFunc(Ch, St);
               if RefResult <> ErrResult then
                 for Cnt := 1 to 10 do                  {***************************}
                   ErrResult := CharPosFunc(Ch, St);    {**  Breakpoint position  **}
             end;                                       {***************************}
  {/Sub} end;

begin
  if StopTesting then exit;
  if LoopStart in [10..15] then s := IntToHex(LoopStart, 1)
  else if LoopStart >= 0  then s := IntToStr(LoopStart)
  else s := '';
  Version := Version + s;
  AddTag(LogList, tgs_FuncVersion, Version);
  AddTag(LogList, tgl_FuncLoopStart, LoopStart);
  for iRun := 1 to RunsSpinEdit.Value do
  begin
    if StopTesting then exit;
    CheckSum := 0;
    inc(CurrentFuncNum);
    if CurrentFuncNum > 2 then
      SetStatus(st_Test, IntToStr(round(CurrentFuncNum * 100 / TotalFuncNum)) +
                ' %    (Doing nothing else now on the computer would improve the accuracy!)');
    { Run Test }
    DebugTimerStart;
    for iString := High(HashList.SortArray) downto 0 do
      for iChar := 1 to Length(SearchMask) do
        inc(CheckSum, CharPosFunc(SearchMask[iChar], HashList.SortArray[iString]^));
    Result := DebugTimerStop;
    if CharPosFunc = @CharPos_SysPosByCall then CharPosStat.ReferenceTime := result;
    { Get function name or run number and begin next memo line generation}
    if iRun = 1 then s := 'Function ' + Version
    else s := '    Run ' + IntToStr(iRun);
    for i := Length(s) to 44 do s := s + ' ';
    { Verify result }
    if (CharPosFunc <> @CharPos_Statistics) and
       (CharPosFunc <> @CharPos_AsmDummy_NoLoop) and
       (Pos('Asm4', Version) = 0) and
       (CheckSum <> CharPosStat.CumulatedResult) then
    begin
      s := s + 'ERROR!';                             // Error detection on deviating results
      if iRun = 1 then AddTag(LogList, tgb_FuncError, true);
      DebugError;                                    // for debugging see subprocedure DebugError
    end
    else begin
      { If okay add total time to the memo line }
      if (CharPosFunc = @CharPos_Statistics) then s := s + 'Reference  '
      else if (CharPosFunc = @CharPos_AsmDummy_NoLoop) or
              (Pos('Asm4', Version) > 0) then s := s + 'Empty      '
           else s := s + 'Verified   ';
      s := s + DebugTimeTotal;
      if iRun = 1 then AddTag(LogList, tgb_FuncError, false);
      AddTag(LogList, tgf_FuncRunTime, Result, iRun);
      { add speed percentage compared to System.Pos(Char;ShortString) }
      if CharPosStat.ReferenceTime > 0.001 then
        s := s + ' (' + format('%5.1f',[Result * 100 / CharPosStat.ReferenceTime]) + ' %)'
      else s := S + '          ';
      { add Time / Call }
      s := s + '  = ' + SecondsToString(Result / CharPosStat.NumCalls, 8) + ' / Call';
      { if it's an non-dummy Asm function add Loop time part / call }
      if (CharPosStat.DummyTime > 0.001) and (Pos('Asm', Version) > 0)
                                         and (Pos('Asm4', Version) = 0) then
      begin
        LoopTimePart := (Result - CharPosStat.DummyTime) / CharPosStat.NumCalls;
        s := s + '  = ' + SecondsToString(LoopTimePart, 8) + ' Loop time / Call';
      end;
    end;
    ResMemo.Append(s);
    if (CharPosFunc = @CharPos_Statistics) or
       (CharPosFunc = @CharPos_AsmDummy_NoLoop) then exit; // no more runs in case of statistics grabbing
  end;
  ResMemo.Append('');
end;

procedure TForm1.FuncGroupTest(Group : TCharPosFuncGroup);
var
  i : integer;
begin
  if StopTesting then exit;
  {$IFNDEF Developing}
  for i := 1 to 2 do ResMemo.Append('');
  ResMemo.Append(Group.Text[0]);
  ResMemo.Append('');
  ResMemo.Append(Group.Text[1]);
  ResMemo.Append('');
  for i := 2 to 11 do ResMemo.Append(Group.Text[i]);
  ResMemo.Append('');
  {$ENDIF}
  for i := 0 to 15 do FuncTest(Group.Func[i], Group.Name, i);
  FuncTest(Group.Func[16], Group.Name, 32);
  FuncTest(Group.Func[17], Group.Name, 64);
end;

procedure TForm1.StartBtnClick(Sender : TObject);
var
  s : String;
  i : integer;
begin
  if IsTesting then
  begin
    StopTesting := true;
    Application.ProcessMessages;
    exit;
  end;
  IsTesting := true;
  DirEdit.Enabled := false;
  RunsSpinEdit.Enabled := false;
  StartBtn.Caption := 'Stop Test';
  CloseBtn.Enabled := false;
  StartTime := Now;
  SetStatus(st_Stat);
  TotalFuncNum := NumStatFuncs +
                 (NumPasFuncs + NumAsmFuncGroups * NumAsmGroupFuncs) * RunsSpinEdit.Value;
  CurrentFuncNum := 0;
  { First grabbing statistics }
  begin
    while StatMemo.Lines.Count > 4 do StatMemo.Lines.Delete(4);   // from previous runs
    ResMemo.Clear;
    LogList.Clear;
    s := SearchMask[1];
    for i := 2 to Length(SearchMask) do s := s + ';' + SearchMask[i];
    StatMemo.Append('Number of search chars:' + IntToSepStr(Length(SearchMask), 33) + '      (' + s + ')');
    ResMemo.Append('');
    ResMemo.Append('***************************  Statistics grabbing functions  **************************');
    ResMemo.Append('');
    Application.ProcessMessages;
    ResetCharPosStats;
    FuncTest(@CharPos_Statistics, 'Statistics');
    CharPosStat.DummyTime := FuncTest(@CharPos_AsmDummy_NoLoop, 'AsmDummy_NoLoop');
    StatMemo.Append('Number of calls per function:            ' +
                    IntToSepStr(CharPosStat.NumCalls, 15));
    s := format('%5.2f',[CharPosStat.Matches * 100 / CharPosStat.NumCalls]) + ' %';
    StatMemo.Append('Number of matches per function:          ' +
                     IntToSepStr(CharPosStat.Matches, 15) + '   =  ' + s);
    s := format('%5.2f',[CharPosStat.Mismatches * 100 / CharPosStat.NumCalls]) + ' %';
    StatMemo.Append('Number of mismatches per function:       ' +
                    IntToSepStr(CharPosStat.Mismatches, 15) + '   =  ' + s);
    StatMemo.Append('Number of Char comparisons per function: ' +
                    IntToSepStr(CharPosStat.CharComparisons, 15));
    StatMemo.Append('Cumulated result per function:           ' +
                    IntToSepStr(CharPosStat.CumulatedResult, 15) + '      (used for verification)');
    s := format('%15.2f',[CharPosStat.CharComparisons / CharPosStat.NumCalls]);
    StatMemo.Append('Average char comparisons per call:       ' + s);
    s := format('%15.2f',[CharPosStat.CumulatedResult / CharPosStat.Matches]);
    StatMemo.Append('Average result per matched call:         ' + s);
  end;
  SetStatus(st_Test);
  { Real function test }
  for i := 1 to 2 do ResMemo.Append('');
  ResMemo.Append('*********************************  Pascal functions  *********************************');
  ResMemo.Append('');
  FuncTest(@CharPos_SysPosByCall, 'SysPosByCall');
  {$IFNDEF Developing}
  FuncTest(@CharPos_SysPosClone_NoAlign, 'SysPosClone_NoAlign');
  FuncTest(@CharPos_SysPosClone_ProcAlign, 'SysPosClone_ProcAlign');
  FuncTest(@CharPos_SysPosClone_ProcLoopAlign, 'SysPosClone_ProcLoopAlign');
  FuncTest(@CharPos_OwnPascal_NoAlign, 'OwnPascal_NoAlign');
  FuncTest(@CharPos_OwnPascal_ProcAlign, 'OwnPascal_ProcAlign');
  FuncTest(@CharPos_OwnPascal_ProcLoopAlign, 'OwnPascal_ProcLoopAlign');
  {$ENDIF}
  for i := 1 to 2 do ResMemo.Append('');
  FuncGroupTest(CharPosFuncGroup_Asm1);
  FuncGroupTest(CharPosFuncGroup_Asm2);
  FuncGroupTest(CharPosFuncGroup_Asm3a);
  FuncGroupTest(CharPosFuncGroup_Asm3b);
  FuncGroupTest(CharPosFuncGroup_Asm3c);
  FuncGroupTest(CharPosFuncGroup_Asm4a);
  FuncGroupTest(CharPosFuncGroup_Asm4b);
  for i := 1 to 2 do ResMemo.Append('');
  ElapsedTime := Now - StartTime;
  ResMemo.Append('Total time elapsed:  ' + TimeToStr(ElapsedTime));
  if StopTesting then Welcome(true)
  else begin
    Welcome(false);
    AddTag(LogList, tgf_StartTime, StartTime);
    AddTag(LogList, tgf_TotalTestTime, ElapsedTime);
    WriteLogFile;
  end;
  DirEdit.Enabled := true;
  RunsSpinEdit.Enabled := true;
  StartBtn.Caption := 'Start Test';
  CloseBtn.Enabled := true;
  if StopTesting then SetStatus(st_CanTest) else SetStatus(st_Done, LogFileName);
  IsTesting := false;
  StopTesting := false;
end;

end.

