
{ Program AlignTest / Unit TestFunctions

  Copyright (C) 2015 Ruediger Walter <rue.walter@web.de>

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


  ************************************************************************************
  *  For better comparability please don't edit this file when sharing the results!  *
  ***********************************************************************************}


unit TestFunctions;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

const
  Revision_TestFunctions = 'r1001-20150308';
  NumStatFuncs      = 2;
  NumPasFuncs       = 7;
  NumAsmFuncGroups  = 7;
  NumAsmGroupFuncs  = 18;

type
  TCharPosFunc = function(c: Char; const s: ShortString): SmallInt;

  TCharPosFuncGroup = record
    Func : Array[0..17] of TCharPosFunc;
    Name : String;
    Text : Array[0..11] of String;
  end;

  TCharPosStat = Record
    CumulatedResult,
    CharComparisons,
    Matches,
    Mismatches,
    NumCalls        : Int64;
    DummyTime,
    ReferenceTime   : Extended;
  end;

var
  CharPosStat : TCharPosStat;

  CharPosFuncGroup_Asm1,
  CharPosFuncGroup_Asm2,
  CharPosFuncGroup_Asm3a,
  CharPosFuncGroup_Asm3b,
  CharPosFuncGroup_Asm3c,
  CharPosFuncGroup_Asm4a,
  CharPosFuncGroup_Asm4b : TCharPosFuncGroup;

  Revision_CharPosAsmFuncs_X86_64,
  Revision_CharPosAsmFuncs_I386    : String;


procedure ResetCharPosStats;

{ Statistics grabbing versions }

function CharPos_Statistics(c: Char; const s: ShortString): SmallInt;
function CharPos_AsmDummy_NoLoop(c: Char; const s: ShortString): SmallInt; assembler; register;

{ Pascal CharPos versions, not grouped }

function CharPos_SysPosByCall(c: Char; const s: ShortString): SmallInt;
function CharPos_SysPosClone_NoAlign(c: Char; const s: ShortString): SmallInt;
function CharPos_OwnPascal_NoAlign(c: Char; const s: ShortString): SmallInt;
function CharPos_SysPosClone_ProcAlign(c: Char; const s: ShortString): SmallInt;
function CharPos_OwnPascal_ProcAlign(c: Char; const s: ShortString): SmallInt;
function CharPos_SysPosClone_ProcLoopAlign(c: Char; const s: ShortString): SmallInt;
function CharPos_OwnPascal_ProcLoopAlign(c: Char; const s: ShortString): SmallInt;


implementation

procedure ResetCharPosStats;
begin
  with CharPosStat do
  begin
    CumulatedResult := 0;
    CharComparisons := 0;
    Matches         := 0;
    Mismatches      := 0;
    NumCalls        := 0;
    DummyTime       := 0;
    ReferenceTime   := 0;
  end;
end;

{**********************************  Pascal functions  **********************************}

function CharPos_SysPosClone_NoAlign(c: Char; const s: ShortString): SmallInt;
var              // Same as System.Pos, except i and result are SmallInt instead of SizeInt
  i : SmallInt;
  pc : pchar;
begin
  pc:=@s[1];
  for i:=1 to length(s) do
  begin
    if pc^=c then
    begin
      result:=i;
      exit;
    end;
    inc(pc);
  end;
  result:=0;
end;

function CharPos_OwnPascal_NoAlign(c : Char; const s : ShortString): SmallInt;
var              // Attempt to get it a bit more assembler-friendly
  LenS,
  CntS : Byte;
  PS   : PChar;
begin
  result := 0;
  LenS   := Length(S);
  CntS   := LenS;
  if CntS = 0 then exit;
  PS := @S[1];
  while PS^ <> c do
  begin
    dec(CntS);
    if CntS = 0 then exit;
    inc(PS);
  end;
  result := LenS - CntS + 1;
end;

{$CODEALIGN PROC=16}

function CharPos_SysPosClone_ProcAlign(c: Char; const s: ShortString): SmallInt;
var              // Same as System.Pos, except i and result are SmallInt instead of SizeInt
  i : SmallInt;  // Exact copy of CharPos_SysPosClone_NoAlign, but with aligned PROC
  pc : pchar;
begin
  pc:=@s[1];
  for i:=1 to length(s) do
  begin
    if pc^=c then
    begin
      result:=i;
      exit;
    end;
    inc(pc);
  end;
  result:=0;
end;

function CharPos_OwnPascal_ProcAlign(c : Char; const s : ShortString): SmallInt;
var               // Exact copy of CharPos_OwnPascal_NoAlign, but with aligned PROC
  LenS,
  CntS : Byte;
  PS   : PChar;
begin
  result := 0;
  LenS   := Length(S);
  CntS   := LenS;
  if CntS = 0 then exit;
  PS := @S[1];
  while PS^ <> c do
  begin
    dec(CntS);
    if CntS = 0 then exit;
    inc(PS);
  end;
  result := LenS - CntS + 1;
end;

{$CODEALIGN LOOP=16}

function CharPos_SysPosByCall(c : Char; const s : ShortString): SmallInt;
begin                           // used as reference for result verification
  Result := Pos(c, s);
  { Which is (in file sstrings.inc):
      function pos(c:char;const s:shortstring):SizeInt;
      var
        i : SizeInt;
        pc : pchar;
      begin
        pc:=@s[1];
        for i:=1 to length(s) do
         begin
           if pc^=c then
            begin
              pos:=i;
              exit;
            end;
           inc(pc);
         end;
        pos:=0;
      end;      }
end;

function CharPos_SysPosClone_ProcLoopAlign(c: Char; const s: ShortString): SmallInt;
var              // Same as System.Pos, except i and result are SmallInt instead of SizeInt
  i : SmallInt;  // Exact copy of CharPos_SysPosClone_NoAlign, but with aligned PROC  & LOOP
  pc : pchar;
begin
  pc:=@s[1];
  for i:=1 to length(s) do
  begin
    if pc^=c then
    begin
      result:=i;
      exit;
    end;
    inc(pc);
  end;
  result:=0;
end;

function CharPos_OwnPascal_ProcLoopAlign(c : Char; const s : ShortString): SmallInt;
var              // Exact copy of CharPos_OwnPascal_NoAlign, but with aligned PROC & LOOP
  LenS,
  CntS : Byte;
  PS   : PChar;
begin
  result := 0;
  LenS   := Length(S);
  CntS   := LenS;
  if CntS = 0 then exit;
  PS := @S[1];
  while PS^ <> c do
  begin
    dec(CntS);
    if CntS = 0 then exit;
    inc(PS);
  end;
  result := LenS - CntS + 1;
end;

{****************************  Statistics grabbing functions  ***************************}

function CharPos_Statistics(c : Char; const s : ShortString): SmallInt;
var
  LenS,
  CntS : Byte;
  PS   : PChar;
begin
  result := 0;
  inc(CharPosStat.NumCalls);                 // inc NumCalls
  LenS   := Length(S);
  CntS   := LenS;
  if CntS = 0 then exit;
  PS := @S[1];
  while PS^ <> c do
  begin
    inc(CharPosStat.CharComparisons);        // inc CharComparisons on mismatch
    dec(CntS);
    if CntS = 0 then
    begin
      inc(CharPosStat.Mismatches);           // inc Mismatches
      exit;
    end;
    inc(PS);
  end;
  result := LenS - CntS + 1;
  inc(CharPosStat.CharComparisons);          // inc CharComparisons on match
  inc(CharPosStat.Matches);                  // inc Matches
  inc(CharPosStat.CumulatedResult, result);  // inc CumulatedResult
end;

{$ifdef CPUX86_64} {$I CharPosAsmFuncs_X86_64.inc}
{$else} {$ifdef CPUI386} {$I CharPosAsmFuncs_i386.inc}
        {$endif}
{$endif}

initialization
  InitFuncGroups;
  {$ifdef CPUX86_64}
  Revision_CharPosAsmFuncs_X86_64 := Revision_CharPosAsm_X86_64_IncFile;
  Revision_CharPosAsmFuncs_I386   := 'Not used';
  {$else}
  Revision_CharPosAsmFuncs_X86_64 := 'Not used';
  Revision_CharPosAsmFuncs_I386   := Revision_CharPosAsm_I386_IncFile;
  {$endif}
end.


